{% extends "pretixcontrol/help/base.html" %}
{% block title %}Payment fee calculation{% endblock %}
{% block inner %}
    <h2>Payment fee calculation</h2>
    <p>
        If you configure a fee for a payment method, there are two possible ways for us to calculate this. Let's
        assume that your payment provider, e.g. PayPal, charges you 5 % fees and you want to charge your users the
        same 5 %, such that for a ticket with a list price of 100 € you will get your full 100 €.
    </p>
    <ul>
        <li>
            <strong>Method A: Calculate the fee from the subtotal and add it to the bill.</strong> For a ticket price of
            100 €, this will lead to the following calculation:
            <table class="table helper-width-auto">
                <tr>
                    <td>Ticket price</td>
                    <td class="text-right">100.00 €</td>
                </tr>
                <tr>
                    <td>pretix calculates the fee as 5% of 100 €</td>
                    <td class="text-right">+ 5.00 €</td>
                </tr>
                <tr>
                    <td>Subtotal that will be paid by the customer</td>
                    <td class="text-right">105.00 €</td>
                </tr>
                <tr>
                    <td>PayPal calculates its fee as 5% of 105 €</td>
                    <td class="text-right">- 5.25 €</td>
                </tr>
                <tr>
                    <td>End total that is on your bank account</td>
                    <td class="text-right"><strong>99.75 €</strong></td>
                </tr>
            </table>
        </li>
        <li>
            <strong>Method B (default): Calculate the fee from the total value including the fee.</strong> For a ticket
            price of 100 €, this will lead to the following calculation:
            <table class="table helper-width-auto">
                <tr>
                    <td>Ticket price</td>
                    <td class="text-right">100.00 €</td>
                </tr>
                <tr>
                    <td>pretix calculates the fee as 100/(100 - 5)% of 100 €</td>
                    <td class="text-right">+ 5.26 €</td>
                </tr>
                <tr>
                    <td>Subtotal that will be paid by the customer</td>
                    <td class="text-right">105.26 €</td>
                </tr>
                <tr>
                    <td>PayPal calculates its fee as 5% of 105.26 €</td>
                    <td class="text-right">- 5.26 €</td>
                </tr>
                <tr>
                    <td>End total that is on your bank account</td>
                    <td class="text-right"><strong>100.00 €</strong></td>
                </tr>
            </table>
            <div class="alert-warning alert">
                Due to the various rounding steps performed by us and by the payment provider, the end total on
                your bank account might stil vary by one cent.
            </div>
        </li>
    </ul>
{% endblock %}