{% extends "pretixcontrol/base.html" %}
{% load i18n %}
{% block title %}{% trans "Dashboard" %}{% endblock %}
{% block content %}
	<h1>{% trans "Dashboard" %}</h1>
    <div class="row dashboard">
        {% for w in widgets %}
            <div class="widget-container widget-{{ w.display_size|default:"small" }}">
                {% if w.url %}
                    <a href="{{ w.url }}" class="widget">
                        {{ w.content|safe }}
                    </a>
                {% else %}
                    <div class="widget">
                        {{ w.content|safe }}
                    </div>
                {% endif %}
            </div>
        {% endfor %}
    </div>
{% endblock %}
