{% load i18n %}

<p>{% blocktrans trimmed %}
    The total amount will be withdrawn from your credit card.
{% endblocktrans %}</p>
<dl class="dl-horizontal">
    <dt>{% trans "Card type" %}</dt>
    <dd>{{ request.session.payment_stripe_brand }}</dd>
    <dt>{% trans "Card number" %}</dt>
    <dd>**** **** **** {{ request.session.payment_stripe_last4 }}</dd>
</dl>

